# coding: utf-8
from __future__ import unicode_literals

from report_writer.core.content.base import BaseContent
from report_writer.core.content.samples.getters import ContextContent


try:
    from typing import TYPE_CHECKING
except ImportError:
    TYPE_CHECKING = False

if TYPE_CHECKING:
    from typing import Any
    from typing import Iterable
    from typing import Optional


class ContentChain(BaseContent):
    """
    Контент описывающий получение данных как последовательное использование
    других объектов контента с передачей им результатов предыдущих шагов.

    .. code-block::

        content = ContentChain([
            KeyContent('foo'),
            TemplateContent('{{ bar }}'),
        ])
        content.render({
            'foo': {'bar': 1}
        })  # '1'

    :param content_iterator: Итератор объектов контента
    """
    def __init__(self,
                 content_iterator,  # type: Iterable[BaseContent]
                 ):  # type: (...) -> None
        super(ContentChain, self).__init__()
        self._content_iterator = (
            content_iterator)  # type: Iterable[BaseContent]

    def render(self,
               context  # type: Any
               ):  # type: (...) -> Any
        current_context = context

        for content in self._content_iterator:
            current_context = content.render(current_context)

        return current_context


class ContentGate(ContentChain):
    """
    Контент описывающий получение данных как последовательное использование
    2 других объектов контента.

    .. code-block::

        content = ContentGate(
            KeyContent('foo'),
            TemplateContent('{{ bar }}'),
        )
        content.render({
            'foo': {'bar': 1}
        })  # '1'

    :param enter_content: Контент "на входе"
    :param exit_content: Контента "на выходе"
    """

    default_base_content = ContextContent()  # type: BaseContent

    def __init__(self,
                 enter_content=None,  # type: Optional[BaseContent]
                 exit_content=None  # type: Optional[BaseContent]
                 ):  # type: (...) -> None
        if enter_content is None:
            enter_content = self.default_base_content

        if exit_content is None:
            exit_content = self.default_base_content

        super(ContentGate, self).__init__((enter_content, exit_content))


class ConditionContent(BaseContent):
    """
    Контент описывающий получение данных по условию.

    В зависимости от исполнения условия будет использован тот или иной
    объект контента.

    .. code-block::

        content = ConditionContent(
            (Content() > 3) & (Content() < 5),
            ContentGate(
                Content() >> (lambda i: {'context': i}),
                TemplateContent('3 < {{ context }} < 5')
            ),
            NoneContent()
        )

        content.render(3)  # None
        content.render(4)  # '3 < 4 < 5'
        content.render(5)  # None

    :param condition: Контент-условие
    :param true: Контент если условие истинно
    :param false: Контент если условие ложно
    """
    def __init__(self,
                 condition,  # type: BaseContent
                 true,  # type: BaseContent
                 false  # type: BaseContent
                 ):
        self._condition = condition  # type: BaseContent
        self._true = true  # type: BaseContent
        self._false = false  # type: BaseContent

    def render(self,
               context  # type: Any
               ):  # type: (...) -> Any
        if self._condition.render(context):
            return self._true.render(context)
        else:
            return self._false.render(context)
